# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Brian Robles brianrobles204@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

"""this dialog adjusts values in gsettings
"""

from gi.repository import Gtk # pylint: disable=E0611
import logging, threading, os, shutil
logger = logging.getLogger('xkcd_browser_lib')

from . helpers import get_builder, show_uri, get_help_uri

class PreferencesDialog(Gtk.Dialog):
    __gtype_name__ = "PreferencesDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated PreferencesDialog object.
        """
        builder = get_builder('PreferencesXkcdBrowserDialog')
        new_object = builder.get_object("preferences_xkcd_browser_dialog")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initalizing should be called after parsing the ui definition
        and creating a PreferencesDialog object with it in order to
        finish initializing the start of the new PerferencesXkcdBrowserDialog
        instance.
        
        Put your initialization code in here and leave __init__ undefined.
        """

        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)

        # code for other initialization actions should be added here

    def on_btn_close_clicked(self, widget, data=None):
        self.destroy()

    def on_btn_help_clicked(self, widget, data=None):
        show_uri(self, "ghelp:%s" % get_help_uri('preferences'))
        
    def on_btn_accounts_clicked(self, widget, data=None):
		self.bca = threading.Thread( target=os.system, args=('gwibber-accounts',))
		self.bca.start()
		
	def on_cache_btn_clicked(self, widget, data=None):
		#Clearing the cache
		cache_dir = os.environ['HOME'] + '/.cache/xkcd-browser'
		
		dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.WARNING, Gtk.ButtonsType.NONE, "Are you sure you want to delete your cache?")
		dialog.add_buttons(Gtk.STOCK_HELP, Gtk.ResponseType.HELP, Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OK, Gtk.ResponseType.OK)
		dialog.set_default_response(Gtk.ResponseType.CANCEL)
		result = dialog.run()
		
		if result == Gtk.ResponseType.OK:
		    try:
				cache_folder = os.listdir(cache_dir)
				cache_folders_for_deletion = filter(lambda x: x.find("_") != -1, cache_folder)
				for folder in cache_folders_for_deletion:
					full_path = cache_dir + "/" + folder
					shutil.rmtree(full_path)
			except:
				# The cache folder does not exist. Our work is done.
				pass
		elif result == Gtk.ResponseType.HELP:
			print "bring up help" #TODO
            show_uri(self, "ghelp:%s" % get_help_uri('cache'))
		else: pass
				
		dialog.destroy()
		

