# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Brian Robles brianrobles204@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from __future__ import division
from gi.repository import Gio, GObject, Gtk, Gdk, GdkPixbuf, Pango, GwibberGtk # pylint: disable=E0611
import os, random, logging, urllib, urllib2, sgmllib, re, threading
logger = logging.getLogger('xkcd_browser_lib')

from . helpers import get_builder, show_uri, get_help_uri


GObject.threads_init()

CACHEDIR = os.environ['HOME'] + '/.cache/xkcd-browser'
GSETTINGS = Gio.Settings("net.launchpad.xkcd-browser")
COMIC = { 'comicImg' : 'empty',
          'comicNo' : 'empty',
          'comicName' : 'empty',
          'comicAlt' : 'empty' }
REQUESTED_URL = "http://xkcd.com"


# Cache class to take care of Favorites, Offline mode, and
# any extra niceties
class CacheClass():
	def __init__(self):
		self.current = CACHEDIR + "/current"
		self.myparser = MyParser()
        if not os.path.exists(CACHEDIR):
			os.makedirs(CACHEDIR)
		if not os.path.exists(self.current):
			os.makedirs(self.current)
		
	def set_from_url(self, url_s, override_cache = False):
		# Read from the internet
		global COMIC
		
		url = urllib.urlopen(url_s)
        source = url.read()
        self.myparser.parse(source)
        comickeys = ['comicImg', 'comicNo', 'comicName', 'comicAlt']
        COMIC = { 'comicImg' : self.myparser.get_img_string(), 
                  'comicNo' : self.myparser.get_comic_no(), 
                  'comicName' : self.myparser.get_comic_name(), 
                  'comicAlt' : self.myparser.get_comic_alt() }
                  
	    if self.myparser.get_latest_comic():
	        cache_latestcomic = self.current + "/latestcomic.txt"
	        with open(cache_latestcomic, 'w') as f:
			    f.write(COMIC['comicNo'])
                
        cache_folder = CACHEDIR + "/x_" + COMIC["comicNo"]
        if os.path.exists(cache_folder) and not override_cache: 
            self.set_from_cache(cache_folder + '/metadata.txt')
            return
		else:
            response = urllib2.urlopen(COMIC['comicImg'])
            image = response.read()
            
            if GSETTINGS.get_value("cache"):
	    	    # We create cache folders and contents here
    	    	cache_folder = CACHEDIR + "/x_" + COMIC['comicNo']
	        	if not os.path.exists(cache_folder):
	        		os.makedirs(cache_folder)
	    	    cache_imgf = cache_folder + "/comic.png"
                with open(cache_imgf,'w') as f:
                    f.write(image)
                cache_meta = cache_folder + "/metadata.txt"
                with open(cache_meta, 'w') as f:
    	    		for key in comickeys:
	        			f.write("%s\n" % COMIC[key])
	    			
			self.set_as_current(image)
			
		
    def set_from_cache(self, fname = CACHEDIR + "/current/metadata.txt"):
		#Function that sets the comic from specified cache directory or from current directory
		global COMIC
		
		raw_comic_obj = [line.strip() for line in open(fname)]
		keys = [ 'comicImg', 'comicNo', 'comicName', 'comicAlt']
		COMIC = dict(zip(keys, raw_comic_obj))
		
		if fname != self.current + "/metadata.txt": self.set_as_current()
				
				
	def set_as_current(self, image=None):
        comickeys = ['comicImg', 'comicNo', 'comicName', 'comicAlt']
		if not image:
			i = open(CACHEDIR + "/x_" + COMIC['comicNo'] + "/comic.png")
		    image = i.read()
		
		# Serve up our current comic in its own folder
		current_imgf = self.current + "/comic.png"
        with open(current_imgf,'w') as f:
            f.write(image)
        current_meta = self.current + "/metadata.txt"
        with open(current_meta, 'w') as f:
			for key in comickeys:
				f.write("%s\n" % COMIC[key])
				
				
	def cache_and_update(self, url_s):
        # Get digits after the link
        dreg = re.compile('(?<=xkcd.com/)\d+')
        m = dreg.search(url_s)
        if m:
			# Regular comic request. Check cache, or get from the internet:
            url_clip = m.group(0)
            cache_folder = CACHEDIR + "/x_" + url_clip
			# Offline mode is handled by the next and previous buttons instead.
			# If there is no internet, it's better to warn the user and teach our feature without force.
            if os.path.exists(cache_folder): self.set_from_cache(cache_folder + '/metadata.txt')
            elif GSETTINGS.get_value("offlinemode"):
				raise CacheError("That comic does not exist.")
				return
			else: self.set_from_url(url_s)
			
        elif url_s.find("random") != -1:
			# Random comic request. Check cache for random comic, or get from the internet.
			if GSETTINGS.get_value("offlinemode"):
				# Offline mode is on, so we load a random comic from the cache.
				all_cache_folders = os.listdir(CACHEDIR)
				comic_folders = []
				for folder in all_cache_folders:
					# leave out possible extra folders, as well as the comic you are currently on (to avoid redundancy)
					if folder.find('x') != -1 and (folder != "x_" + COMIC['comicNo'] or len(all_cache_folders) == 2):
						comic_folders.append(folder)
				try: comic_dir = CACHEDIR + "/" + comic_folders[int(random.random() * len(comic_folders))]
				except:
					raise CacheError("Your cache is empty.")
					return
				self.set_from_cache(comic_dir + '/metadata.txt')
			else: self.set_from_url(url_s)
			
		elif url_s == "cache://lastaccessed":
			# Last accessed comic request. Use the comic in the current folder, otherwise load the latest from cache or internet.
			cache_img = self.current + "/comic.png"
			cache_meta = self.current + "/metadata.txt"
			if os.path.exists(cache_img) and os.path.exists(cache_meta):
				self.set_from_cache()
			else: self.set_last_and_update()
			
		elif url_s == "cache://latestcomic": self.set_last_and_update() # For your readability ;)
		else: self.set_last_and_update()
	    	
	    	
    def url_cache_and_update(self, url_s):
		# This is an emergency function that parses the url and calls the appropriate webpage via internet only.
		# To be used if the local cache is somehow broken. We update it fresh from the internet.
		if not GSETTINGS.get_value("offlinemode"):
            dreg = re.compile('(?<=xkcd.com/)\d+') # Get digits after the link
            m = dreg.search(url_s)
            if m or url_s == "http://dynamic.xkcd.com/random/comic/":
	    		# Regular/Random comic request. Get directly from the internet:
	    		self.set_from_url(url_s, True)
	    	else: 
	    	    # Built in requests for "last accessed" comic or "latest available comic" will simply load the latest page.
	    	    self.set_from_url("http://xkcd.com", True)
	    else:
			raise IOError
			
		
	def set_last_and_update(self):
		# Last possible comic request. Load the last comic from cache in Offline mode, or simply load the latest from the internet.
		if GSETTINGS.get_value("offlinemode"):
			# Offline mode is on, so we load the last comic from the cache
			all_cache_folders = os.listdir(CACHEDIR)
			new_cache_folders = sorted(filter(lambda x: x.find("_") != -1, all_cache_folders), key = lambda x: int(x.split("_")[1]))
			try: comic_dir = CACHEDIR + "/" + new_cache_folders[len(new_cache_folders)-1]
			except:
				raise CacheError("Your cache is empty.")
				return
			self.set_from_cache(comic_dir + '/metadata.txt')
		else: self.set_from_url("http://xkcd.com")
		
		
	def isBool(self, string):
		return string[0].upper() == 'T'
            
    def get_image(self):
		return self.current + "/comic.png"
		
		

class CacheError(Exception):
    def __init__(self, value):
        Exception.__init__(self, "Cache Error: %s" % value)
        self.info = value
    def __str__(self):
		return self.info
		

# A class to parse xkcd links and find the hotlink
# for our needed image
class MyParser(sgmllib.SGMLParser):
    def parse(self, s):
        self.latestcomic = False
		self.source = s
        self.feed(s)
        self.close()

    def __init__(self, verbose=0):
        sgmllib.SGMLParser.__init__(self, verbose)
        self.linktoimage = None
        self.imagetitle = None

    def start_img(self, attributes):
        for name, value in attributes:
            if name == "src" and value.find("imgs.xkcd.com/comics") != -1:
                self.linktoimage = value
            if name == "title":
				self.imagetitle = value
				
	def start_a(self, attributes):
		for name, value in attributes:
			if name == "href" and value == "#":
				self.latestcomic = True

    def get_img_string(self):
        return self.linktoimage
        
    def get_comic_no(self):
        #match the group of digits following the string
        reg = re.compile('(?<=Permanent link to this comic: http://xkcd.com/)\d+')
        comicno = reg.search(self.source)
        return comicno.group(0)
        
    def get_comic_name(self):
        #match everything following and being followed by the two title tag bits
        reg = re.compile('(?<=<title>xkcd: ).*?(?=</title>)')
        comicname = reg.search(self.source)
        return comicname.group(0)
        
    def get_comic_alt(self):
        return self.imagetitle
        
    def get_latest_comic(self):
		return self.latestcomic
		


class image_thread(threading.Thread):
	def __init__(self, xkcdbrowser, url):
	    super(image_thread, self).__init__()
	    self._stop = threading.Event()
	    self.cache = CacheClass()
	    #use the right references
	    self.url_s = url
	    self.xb = xkcdbrowser
	    self.img_main = self.xb.img_main
	    self.titlelabel = self.xb.titlelabel
	    self.next_button = self.xb.next_button
	    self.last_button = self.xb.last_button
	    self.tbtn_alt = self.xb.tbtn_alt
	    self.eventbox_main = self.xb.eventbox_main
	    self.comic_no_box = self.xb.comic_no_box
	    self.spinner = self.xb.spinner
	    self.info_label = self.xb.info_label
	    self.info_secondlbl = self.xb.info_secondlbl
		
	def stop(self):
		self._stop.set()
		
	def stopped(self):
		return self._stop.isSet()
	    
	def run(self):
		#Open and parse the url string
		if self.stopped(): return
		try: 
		    self.cache.cache_and_update(self.url_s)
		except (IOError, AttributeError, CacheError), e:
			GObject.idle_add(self.update_error, e)
			return
		except:
			GObject.idle_add(self.update_error)
			return
        
		if self.stopped(): return
		GObject.idle_add(self.update_app, self.cache.get_image())
		
	def update_app(self, fname):
		#Load the image and tooltip
        self.img_main.set_from_file(fname)
        self.img_main.set_tooltip_text(COMIC['comicAlt'])
		self.img_main.set_has_tooltip(not self.tbtn_alt.get_active())
        #Change the title
        self.titlelabel.set_text('xkcd ' + COMIC['comicNo'] + ': ' + COMIC['comicName'])
        self.xb.set_title('xkcd ' + COMIC['comicNo'] + ': ' + COMIC['comicName'])
        #Miscellaneous stuff
        self.xb.update_button_sensitivity(COMIC['comicNo'])
        self.xb.prepare_image_zoom()
        self.xb.isloading = False
        self.comic_no_box.set_placeholder_text(COMIC['comicNo'])
		#Finalize
		self.eventbox_main.set_visible(True)
		self.spinner.set_visible(False)
		self.spinner.stop()
		
	def update_error(self, error_obj = "[Unkown Error]"):
		if str(error_obj) == "'NoneType' object has no attribute 'group'":
			# There's a 404 on the website. Tell the user to load another comic.
			button_dict = { 'Dismiss' : 0 }
			self.xb.error_info_create("Connection Error", "The app can't connect to the website. Try loading another comic.", button_dict)
			
		elif str(error_obj) == "[Errno socket error] [Errno -2] Name or service not known":
			# There's something wrong with the user's internet. Suggest offline mode.
		    button_dict = { 'Go Offline' : 1, 'Dismiss' : 0 }
			self.xb.error_info_create("Network Error", "Your internet seems down. Try another comic, or try using Offline mode.", button_dict)
		
		elif str(error_obj).find("[Errno 2] No such file or directory:") != -1:
			# There is something wrong with the cache. Try loading directly from the internet. No need to warn the user.
			button_dict = { 'Dismiss' : 0 }
			try:
		        if self.stopped(): return
			    self.cache.url_cache_and_update(self.url_s)
		        if self.stopped(): return
		        GObject.idle_add(self.update_app, self.cache.get_image())
			except:
			    self.xb.error_info_create("Error", "Something went wrong. Check your internet. Check that Offline mode is off. Try clearing your cache.", button_dict)
			
		elif str(error_obj) == "Your cache is empty.":
		    button_dict = { 'Go Online' : 2, 'Dismiss' : 0 }
		    self.xb.error_info_create("Cache Error", "You have no comics in your cache. Try going online instead.", button_dict)
		    
		elif str(error_obj) == "That comic does not exist.":
		    button_dict = { 'Go Online' : 2, 'Dismiss' : 0 }
		    self.xb.error_info_create("Cache Error", "That comic does not exist in your cache. Try going online instead.", button_dict)
			
		else:
			# There is some unspecified error. Tell the user to try restarting the app.
			button_dict = { 'Dismiss' : 0 }
			self.xb.error_info_create("Error", "Something went wrong. Try restarting the app. More info: " + str(error_obj), button_dict)
        self.xb.isloading = False
			
		

# This class is meant to be subclassed by XkcdBrowserWindow.  It provides
# common functions and some boilerplate.
class Window(Gtk.Window):
    __gtype_name__ = "Window"

    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your initialization code in finish_initializing
    
    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated BaseXkcdBrowserWindow object.
        """
        builder = get_builder('XkcdBrowserWindow')
        new_object = builder.get_object("xkcd_browser_window")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initializing should be called after parsing the UI definition
        and creating a XkcdBrowserWindow object with it in order to finish
        initializing the start of the new XkcdBrowserWindow instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)
        self.PreferencesDialog = None # class
        self.preferences_dialog = None # instance
        self.AboutDialog = None # class
        self.isloading = False
        
        self.window_box = self.builder.get_object('window_box')
        self.img_main = self.builder.get_object('img_main')
        self.eventbox_main = self.builder.get_object('eventbox_main')
        self.next_button = self.builder.get_object('tbtn_next')
        self.last_button = self.builder.get_object('tbtn_last')
        self.prev_button = self.builder.get_object('tbtn_previous')
        self.tbtn_alt = self.builder.get_object('tbtn_alt')
        self.tbtn_offline = self.builder.get_object('tbtn_offline')
        self.comic_no_box = self.builder.get_object('comic_no_box')
        
        self.scrolledwindow = self.builder.get_object('scrolledwindow')
        self.spinner = self.builder.get_object('spinner')
        
        self.event_altbox = self.builder.get_object('event_altbox')
        self.altlabel = self.builder.get_object('altlabel')
        self.titlelabel = self.builder.get_object('titlelabel')
        
        self.info_label = self.builder.get_object('info_label')
        self.info_secondlbl = self.builder.get_object('info_secondlbl')
        self.info_hbox = self.builder.get_object('info_hbox')
        
        self.mnu_prev = self.builder.get_object('mnu_prev')
        self.mnu_prev.connect("activate", self.arrow_click, "Previous")
        self.mnu_next = self.builder.get_object('mnu_next')
        self.mnu_next.connect("activate", self.arrow_click, "Next")
        
        self.eventbox_main.drag_source_set(Gdk.ModifierType.BUTTON1_MASK, [], Gdk.DragAction.COPY)
        self.eventbox_main.drag_source_add_uri_targets()

        self.settings = GSETTINGS
        self.settings.connect('changed', self.on_preferences_changed)

        # Optional Launchpad integration
        # This shouldn't crash if not found as it is simply used for bug reporting.
        # See https://wiki.ubuntu.com/UbuntuDevelopment/Internationalisation/Coding
        # for more information about Launchpad integration.
        try:
            from gi.repository import LaunchpadIntegration # pylint: disable=E0611
            LaunchpadIntegration.add_items(self.ui.helpMenu, 1, True, True)
            LaunchpadIntegration.set_sourcepackagename('xkcd-browser')
        except ImportError:
            pass

        # Optional application indicator support
        # Run 'quickly add indicator' to get started.
        # More information:
        #  http://owaislone.org/quickly-add-indicator/
        #  https://wiki.ubuntu.com/DesktopExperienceTeam/ApplicationIndicators
        try:
            from xkcd_browser import indicator
            # self is passed so methods of this class can be called from indicator.py
            # Comment this next line out to disable appindicator
            self.indicator = indicator.new_application_indicator(self)
        except ImportError:
            pass
            
        #Load the comic for today
        self.start_image("cache://lastaccessed")
        

    def on_mnu_contents_activate(self, widget, data=None):
        show_uri(self, "ghelp:%s" % get_help_uri())

    def on_mnu_about_activate(self, widget, data=None):
        """Display the about box for xkcd-browser."""
        if self.AboutDialog is not None:
            about = self.AboutDialog() # pylint: disable=E1102
            response = about.run()
            about.destroy()

    def on_mnu_preferences_activate(self, widget, data=None):
        """Display the preferences window for xkcd-browser."""

        """ From the PyGTK Reference manual
           Say for example the preferences dialog is currently open,
           and the user chooses Preferences from the menu a second time;
           use the present() method to move the already-open dialog
           where the user can see it."""
        if self.preferences_dialog is not None:
            logger.debug('show existing preferences_dialog')
            self.preferences_dialog.present()
        elif self.PreferencesDialog is not None:
            logger.debug('create new preferences_dialog')
            self.preferences_dialog = self.PreferencesDialog() # pylint: disable=E1102
            self.preferences_dialog.connect('destroy', self.on_preferences_dialog_destroyed)
            self.preferences_dialog.show()
        # destroy command moved into dialog to allow for a help button

    def on_mnu_close_activate(self, widget, data=None):
        """Signal handler for closing the XkcdBrowserWindow."""
        self.destroy()
        
        
    # Help Menu Buttons
    def on_mnu_visit_activate(self, widget, data=None):
		try: os.system('xdg-open http://xkcd.com')
		except: pass
		
    def on_mnu_aboutweb_activate(self, widget, data=None):
		try: os.system('xdg-open http://xkcd.com/about/')
		except: pass
		
		
	# Comic Menu Buttons
	def on_mnu_save_as_activate(self, widget, data=None):
		dialog = Gtk.FileChooserDialog("Save Comic to File", self, Gtk.FileChooserAction.SAVE, [Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT])
		dialog.set_modal(True)
		dialog.set_default_response(Gtk.ResponseType.ACCEPT)
		dialog.set_current_folder(os.environ['HOME'] + "/Pictures")
		dialog.set_current_name(COMIC['comicName'] + ".png")
		dialog.set_do_overwrite_confirmation(True)
		if dialog.run() == Gtk.ResponseType.ACCEPT:
		    i = open(CACHEDIR + "/current/comic.png")
		    image = i.read()
		    filename = dialog.get_filename()
            with open(filename,'w') as f:
                f.write(image)
        else:
			pass
		dialog.destroy()
		
	def on_mnu_search_activate(self, widget, data=None):
		self.comic_no_box.grab_focus()
		

    def on_destroy(self, widget, data=None):
        """Called when the XkcdBrowserWindow is closed."""
        # Clean up code for saving application state should be added here.
        Gtk.main_quit()

    def on_preferences_changed(self, settings, key, data=None):
        logger.debug('preference changed: %s = %s' % (key, str(settings.get_value(key))))
        if key == "offlinemode":
			if settings.get_value("offlinemode"): self.tbtn_offline.set_stock_id("gtk-disconnect")
			else: self.tbtn_offline.set_stock_id("gtk-connect")
			self.update_button_sensitivity(COMIC['comicNo'])
		if key == "cache":
			if not settings.get_value("cache"): settings.set_boolean("offlinemode", False)

    def on_preferences_dialog_destroyed(self, widget, data=None):
        '''only affects gui
        
        logically there is no difference between the user closing,
        minimising or ignoring the preferences dialog'''
        logger.debug('on_preferences_dialog_destroyed')
        # to determine whether to create or present preferences_dialog
        self.preferences_dialog = None
			
			
	#Handle exploring comics with keypresses
	def keypress_move(self, widget, event):
		if self.comic_no_box.get_text_length() == 0:
			if event.keyval == Gdk.KEY_Left:
				if self.prev_button.get_sensitive(): self.arrow_click(widget, "Previous")
				return True
			elif event.keyval == Gdk.KEY_Right:
				if self.next_button.get_sensitive(): self.arrow_click(widget, "Next")
				return True
        
    
    #Start the thread for loading our image
    def start_image(self, url, requested_comic_no=False):
		global REQUESTED_URL
		REQUESTED_URL = url
		#Check if there are already threads
        if threading.activeCount() > 1:
			for x in range(1, threading.activeCount()):
			    threading.enumerate()[x].stop()
			    threading.enumerate()[x].join()
		#Switch to loading placeholders
        self.eventbox_main.set_visible(False)
        self.spinner.set_visible(True)
        self.spinner.start()
        if requested_comic_no:
			self.titlelabel.set_text('xkcd ' + requested_comic_no + ': ...')
            self.update_button_sensitivity(requested_comic_no)
            self.comic_no_box.set_placeholder_text(requested_comic_no)
		else:
			self.titlelabel.set_text('xkcd Browser...')
            self.comic_no_box.set_placeholder_text('')
        self.set_title('xkcd Browser')
		self.prev_button.grab_focus()
        self.comic_no_box.set_text('')
        self.tbtn_alt.grab_focus()
        #Scrolled window stuff
        swh = self.scrolledwindow.get_hadjustment()
        swh.set_value(0.0)
        swv = self.scrolledwindow.get_vadjustment()
        swv.set_value(0.0)
        self.isloading = True
        #Load image in separate thread
	    it = image_thread(self, url)
        it.start()
			
			
	# Image Zooming
	def prepare_image_zoom(self):
		self.extraimage = self.img_main.get_pixbuf()
		self.i_width, self.i_height = self.extraimage.get_width(), self.extraimage.get_height()
		self.adj_width, self.adj_height = self.extraimage.get_width(), self.extraimage.get_height()
		
	def on_mnu_zoomout_activate(self, widget, data=None):
		self.adj_width = self.adj_width - (self.i_width * .10)
		self.adj_height = self.adj_height - (self.i_height * .10)
		new_image = self.extraimage.scale_simple(self.adj_width, self.adj_height, GdkPixbuf.InterpType.BILINEAR)
		self.img_main.set_from_pixbuf(new_image)
		
	def on_mnu_zoomin_activate(self, widget, data=None):
		self.adj_width = self.adj_width + (self.i_width * .10)
		self.adj_height = self.adj_height + (self.i_height * .10)
		new_image = self.extraimage.scale_simple(self.adj_width, self.adj_height, GdkPixbuf.InterpType.BILINEAR)
		self.img_main.set_from_pixbuf(new_image)
		
	def on_mnu_nsize_activate(self, widget, data=None):
		self.adj_width = self.i_width
		self.adj_height = self.i_height
		new_image = self.extraimage.scale_simple(self.adj_width, self.adj_height, GdkPixbuf.InterpType.BILINEAR)
		self.img_main.set_from_pixbuf(new_image)
		    
    
    # Drag and Drop
    def on_eventbox_main_drag_begin(self, widget, context=None):
		pixbuf = self.img_main.get_pixbuf()
		min_dimension = 100 #Neither the  width nor height should exceed this.
		if self.i_width >= self.i_height:
			new_width = min_dimension
			new_height = int(round(self.i_height * (new_width / self.i_width)))
		else:
			new_height = min_dimension
			new_width = int(round(self.i_width * (new_height / self.i_height)))
		new_pixbuf = pixbuf.scale_simple(new_width, new_height, GdkPixbuf.InterpType.BILINEAR)
        Gtk.drag_set_icon_pixbuf(context, new_pixbuf, int(new_width/2), int(new_height/2))
        
    def on_eventbox_main_drag_data_get(self, widget, drag_context, data, info, time):
		img = "file://" + CACHEDIR + "/current/comic.png"
		data.set_uris([img, None])
		
        
    # Shared functions
    def refresh_image(self, widget, data=None):
        if not self.isloading: self.start_image(REQUESTED_URL)
        
    def open_in_browser(self, widget, data=None):
		try: os.system('xdg-open http://xkcd.com/' + COMIC['comicNo'])
		except: pass
		
	def share_comic(self, widget, data=None):
		dialog = Gtk.Dialog()
		dialog.resize(450, 280)
		dialog.set_title("Share this comic")
		
		share_box = Gtk.HBox()
		
		img = Gtk.Image()
		min_dimension = 100
		if self.i_width >= self.i_height:
			new_width = min_dimension
			new_height = int(round(self.i_height * (new_width / self.i_width)))
		else:
			new_height = min_dimension
			new_width = int(round(self.i_width * (new_height / self.i_height)))
		new_comic = self.extraimage.scale_simple(new_width, new_height, GdkPixbuf.InterpType.BILINEAR)
		img.set_from_pixbuf(new_comic)
		share_box.pack_start(img, True, True, 0)
		img.set_padding(12, 12)
		
		titlemarkup = "<span size=\"medium\" weight=\"bold\">" + "xkcd " + COMIC['comicNo'] + ": " + COMIC['comicName'] + "</span>"
		
		comicinfo = Gtk.VBox()
		comictitle = Gtk.Label()
		comictitle.set_markup(titlemarkup)
		comictitle.set_alignment(0, 0.5)
		comictitle.set_line_wrap(True)
		comictitle.set_size_request(300, -1)
		comicinfo.pack_start(comictitle, True, True, 0)
		comicalt = Gtk.Label()
		comicalt.set_text(COMIC['comicAlt'])
		comicalt.set_alignment(0, 0.5)
		comicalt.set_size_request(300, -1)
		comicalt.set_line_wrap(True)
		comicalt.set_sensitive(False)
		comicinfo.pack_start(comicalt, True, True, 0)
		share_box.pack_start(comicinfo, True, True, 0)
		
		entry = GwibberGtk.Entry()
		entry.set_margin_left(5)
		entry.set_margin_right(5)
		entrybuffer = entry.text_view.get_buffer()
		entrybuffer.set_text("Great comic. xkcd.com/" + COMIC['comicNo'])
		entrybuffer.select_range(entrybuffer.get_iter_at_offset(0), entrybuffer.get_iter_at_offset(12))
		dialog_content = dialog.get_content_area()
		dialog_content.pack_start(share_box, False, True, 5)
		dialog_content.pack_start(entry, True, True, 0)
		dialog.show_all()
		dialog.run()
		dialog.destroy()
        
        
    # Toolbar Buttons
    def arrow_click(self, widget, data=None):
		if data: req_comic_no = self.get_requested_comic_no(data)
		else: req_comic_no = self.get_requested_comic_no(widget.get_label())
		if not self.isloading: 
		    self.start_image("http://xkcd.com/" + str(req_comic_no), str(req_comic_no))
            COMIC['comicNo'] = str(req_comic_no)
		
    def on_tbtn_random_clicked(self, widget, data=None):
        if not self.isloading: self.start_image("http://dynamic.xkcd.com/random/comic/")
        
    def on_tbtn_last_clicked(self, widget, data=None):
        if not self.isloading: self.start_image("cache://latestcomic")
        
    def on_tbtn_alt_toggled(self, widget, data=None):
		self.event_altbox.set_visible(widget.get_active())
		self.img_main.set_has_tooltip(not widget.get_active())
		
		
	# Comic No Box search
	def on_comic_no_box_icon_press(self, widget, icon, event):
		if widget.get_text() == (''): widget.set_text(comicNo['comicNo'])
		self.on_comic_no_box_activate(widget)
		
	def on_comic_no_box_activate(self, widget, data=None):
		raw_req_comic = widget.get_text()
		req_comic = filter(lambda x: x.isdigit(), raw_req_comic)
		if not self.isloading: self.start_image("http://xkcd.com/" + req_comic)
		
	def on_comic_no_box_focus_in_event(self, widget, data=None):
		widget.set_text(COMIC['comicNo'])
		widget.select_region(0, -1)
		return
		
	def on_eventbox_main_button_press_event(self, widget, event, data=None):
		self.prev_button.grab_focus()
		self.comic_no_box.set_text('')
		
		
	def get_requested_comic_no(self, direction):
		button_dict = { 'Go Online' : 2, 'Dismiss' : 0 }
		if GSETTINGS.get_value("offlinemode"):
			try:
				cache_folder = os.listdir(CACHEDIR)
				if len(cache_folder) >= 2:
					# Sort cache and get next one
			        current_comic = "x_" + COMIC['comicNo']
			        try: cache_folder.index(current_comic)
					except: cache_folder.append(current_comic)
			        sorted_cache_folder = sorted(filter(lambda x: x.find("_") != -1, cache_folder), key = lambda x: int(x.split("_")[1]))
			        try:
			            if direction == "Next": req_comic_no = sorted_cache_folder[sorted_cache_folder.index(current_comic) + 1].split("_")[1]
						else:
							req_comic_no = sorted_cache_folder[sorted_cache_folder.index(current_comic) - 1].split("_")[1]
							if req_comic_no < 0: req_comic_no = int(COMIC['comicNo'])
			        except: 
			            # If there is only one in cache, load that one
			            req_comic_no = int(COMIC['comicNo'])
				else: 
				    # If the cache folder is technically empty, raise error
		            self.error_info_create("Cache Error", "You have no comics in your cache. Try going online instead", button_dict)
		            return
			except:
				# If there is no cache folder, raise error
		        self.error_info_create("Cache Error", "You have no comics in your cache. Try going online instead", button_dict)
		        return
		else:
			# If online mode, load the next comic.
		    if direction == "Next":
				req_comic_no = int(COMIC['comicNo']) + 1
			else:
				req_comic_no = int(COMIC['comicNo']) - 1
				
		return req_comic_no
		
		
	def update_button_sensitivity(self, req_comic_no):
		# Function to disable or enable the next, last, and previous buttons according to context.
		button_dict = { 'Go Online' : 2, 'Dismiss' : 0 }
		if GSETTINGS.get_value("offlinemode"):
			try:
				cache_folder = os.listdir(CACHEDIR)
				if len(cache_folder) >= 2:
					# Sort cache and get next one
			        sorted_cache_folder = sorted(filter(lambda x: x.find("_") != -1, cache_folder), key = lambda x: int(x.split("_")[1]))
					try:
						isfirst = (sorted_cache_folder.index("x_" + req_comic_no) == 0)
					    islast = (sorted_cache_folder.index("x_" + req_comic_no) == (len(sorted_cache_folder) - 1))
					except:
						# The current comic isn't in the cache for some reason. Just ignore for now.
						# The navigational buttons can still handle possible requests without fatal errors.
						isfirst = False
						islast = False
					self.next_button.set_sensitive(not islast)
					self.last_button.set_sensitive(not islast)
					self.prev_button.set_sensitive(not isfirst)
				else: 
				    # The cache folder is technically empty. Raise error
		            self.error_info_create("Cache Error", "You have no comics in your cache. Try going online instead", button_dict)
		            return
			except:
				# The cache folder does not exist. Raise error
		        self.error_info_create("Cache Error", "You have no comics in your cache. Try going online instead", button_dict)
		        return
		else:
			# Online mode is on, so we simply block the first and last comic.
			isfirst = (req_comic_no == "1")
			latestcomic = CACHEDIR + "/current/latestcomic.txt"
			if os.path.exists(latestcomic):
				with open(latestcomic) as f:
					latestComicNo = f.readlines()[0]
					islast = (req_comic_no == latestComicNo)
					self.next_button.set_sensitive(not islast)
			else:
				self.next_button.set_sensitive(True)
			self.prev_button.set_sensitive(not isfirst)
			self.last_button.set_sensitive(True)
			
			
	# Error Handling
	def error_info_create(self, label, supplabel, button_dict):
		try: 
		    self.infobar
		    self.error_info_destroy()
		except:
			pass
			
		if self.spinner.get_visible():
			# Show a sad face, but only if there's no comic currently being displayed.
		    self.img_main.set_from_icon_name('face-sad', 1)
		    self.img_main.set_pixel_size(80)
		    self.eventbox_main.set_visible(True)
		    self.spinner.set_visible(False)
		    self.spinner.stop()
		    
		self.infobar = Gtk.InfoBar()
        self.infobar.set_visible(True)
	    self.window_box.pack_start(self.infobar, False, True, 0)
	    self.window_box.reorder_child(self.infobar, 2)
	    self.infobar.connect("response", self.error_info_action)
		
	    self.infobar.set_message_type(Gtk.MessageType.ERROR)
	    self.infobar.set_orientation(Gtk.Orientation.HORIZONTAL)
	    infobar_content = self.infobar.get_content_area()
	    infobar_content.pack_start(self.info_hbox, True, True, 0)
		    
	    for key in button_dict:
	    	self.infobar.add_button(key, button_dict[key])
		
		self.info_label.set_text(label)
		self.info_secondlbl.set_text(supplabel)
			
		
	def error_info_action(self, widget, response_id):
		if response_id == 0:
			# Default action. Close InfoBar
		    self.error_info_destroy()
		if response_id == 1:
			# Reserved for Go Offline button
			GSETTINGS.set_boolean("offlinemode", True)
			self.start_image("cache://lastaccessed")
			self.error_info_destroy()
		if response_id == 2:
			# Reserved for Go Online button
			GSETTINGS.set_boolean("offlinemode", False)
			self.start_image(REQUESTED_URL)
			self.error_info_destroy()
		
	def error_info_destroy(self, widget=None, data=None):
		try:
			content_area = self.infobar.get_content_area()
			content_area.remove(self.info_hbox)
			self.infobar.destroy()
			self.infobar = None
		except:
			pass
                
        
    # Alternate View
    def on_event_altbox_enter(self, widget, data=None):
        self.altlabel.set_text(COMIC['comicAlt'])
        self.altlabel.set_sensitive(True)
        
    def on_event_altbox_leave(self, widget, data=None):
        self.altlabel.set_text("Hover for alternate text.")
        self.altlabel.set_sensitive(False)
